#ifndef SCATTERDB_H
#define SCATTERDB_H

// --8<--8<--8<--8<--
//
// Copyright (C) 2006 Smithsonian Astrophysical Observatory
//
// This file is part of scatter
//
// scatter is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// scatter is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the
//       Free Software Foundation, Inc.
//       51 Franklin Street, Fifth Floor
//       Boston, MA  02110-1301, USA
//
// -->8-->8-->8-->8--


#include <string>
#include <vector>

#include <Exception/Exception.h>

namespace Scatter {

    class ScatterDB {
    public:
	ScatterDB( const std::string& filename ) throw ( Exception ) : _file( filename ) { }

	virtual ~ScatterDB( ) { };

	virtual bool lookUpScatterAngle( double energy, double sin_alpha, double rannum,
					 double& theta ) = 0;


	long nRows( void ) const { return _nrows; };


	static double interpolate( double x, double xa, double ya, double xb, double yb );

	typedef std::vector<double> ColData;

	static bool find_bounds( ColData &data, double value,
				 pair<ColData::iterator, ColData::iterator>& range,
				 pair<long, long>& idx
	    );

	static ScatterDB* load( const std::string& filename );

	virtual bool is_symmetric( ) = 0;

    protected:

	std::string _file;

	long _nrows;

	struct Column2D  {

	    size_t ncols;
	    std::vector<double> data;

	    double& operator ()( const long row, const long col )  { return data[ row * ncols + col ] ;}

	};

    };

}

#endif // !SCATTERDB_H
